/*************************************************************

(C) ISEP 2004
(C) Professores de ADAV 2004

O ISEP, na qualidade de autor do presente software, garante 
o direito de utilizao, alterao, evoluo e distribuio 
do mesmo sem qualquer tipo de restries, excepto:
	- esta mensagem no pode ser retirada
	- os nomes das classes no podem ser alterados

*************************************************************/

using System;
using System.Data;
using System.Data.OleDb;

using AdavSms;
using Isep.Util.Data;

namespace AdavSmsService
{
	/// <summary>
	/// Summary description for AdavSmsImpl.
	/// </summary>
	internal class AdavSmsImpl : IAdavSms
	{
		public AdavSmsImpl()
		{
			//
			// TODO: Add constructor logic here
			//
		}

		#region IAdavSms Members

		public float Send(string token, string destination, string message)
		{
			//
			// validaes
			//
			if (token == null || token == String.Empty)
				throw new ArgumentException("cannot be null or empty", "token");
			if (destination == null || destination == String.Empty)
				throw new ArgumentException("cannot be null or empty", "destination");
			if (message == null || message == String.Empty)
				throw new ArgumentException("cannot be null or empty", "message");
//			if (message.Length > AdavSms.Constants.MAX_LEN_SMS)
//				throw new ArgumentException("must not exceed " + AdavSms.Constants.MAX_LEN_SMS.ToString() + " characteres", "message");
//			if (!IsValidToken(token))
//				throw new ArgumentException("unknown or invalid", "token");

			//
			// processamento
			//
			OleDbConnection cnx = null;
			float cost = 0.0f;
			try
			{
				// abrir conexo
				cnx = OleDbHelper.CreateOpenConnection(AdavSmsService.Constants.CONN_STR);
				
				// validar token e obter custo 
				DataSet ds = OleDbHelper.ExecuteDataSet(cnx, "SELECT Users.id, rate FROM Users, Rates WHERE token='" + token + "' AND Users.id_rate = Rates.id");
				if (ds.Tables[0].Rows.Count != 1)
					throw new ArgumentException("unknown or invalid", "token");
				int userid = (int)ds.Tables[0].Rows[0][0];
				float rate = (float)ds.Tables[0].Rows[0][1];
				/*
				 * outra implementao - menos eficiente
				 */ 
				//				DataSet ds = OleDbHelper.ExecuteDataSet(cnx, "SELECT id, id_rate FROM Users WHERE token='" + token + "'");
				//				if (ds.Tables[0].Rows.Count != 1)
				//					throw new ArgumentException("unknown or invalid", "token");
				//				int userid = (int)ds.Tables[0].Rows[0][0];
				//				int rateid = (int)ds.Tables[0].Rows[0][1];
				//
				//				rate = (float)OleDbHelper.ExecuteScalar(cnx, "SELECT rate FROM Rates WHERE id = " + rateid.ToString());


				//
				//registar mensagem 
				//- garantir 160 caracteres max. por mensagem
				//
				OleDbCommand cmd = new OleDbCommand("INSERT INTO SentMessages (id_user, destination, dateSent, message, cost) VALUES (?, ?, ?, ?, ?)", cnx);
				cmd.Parameters.Add("userid", userid);
				cmd.Parameters.Add("destination", destination);
				OleDbParameter p = cmd.Parameters.Add("dateSent", OleDbType.Date); // parametros do tipo data tm que ser criados explicitamente 
				p.Value = DateTime.Now;
				p = cmd.Parameters.Add("message", OleDbType.Char);
				cmd.Parameters.Add("cost", rate);

				string msg;
				int i;
				for (i = 0; i < message.Length / AdavSms.Constants.MAX_LEN_SMS; i++)
				{
					msg = message.Substring(i*160, 160);
					p.Value = msg;
					OleDbHelper.ExecuteUpdate(cnx, cmd);

					cost += rate;
				}
				msg = message.Substring(i*160, message.Length - i*160);
				p.Value = msg;
				OleDbHelper.ExecuteUpdate(cnx, cmd);
				cost += rate;

				/*
				 * outra implementao - sem usar parametros
				 * de notar que  preciso concatenar strings e ter algum cuidado com os campos
				 * que na BD sejam do tipo string (ver message) ou single (ver rate)
				 */
				//				string sql = "INSERT INTO SentMessages (id_user, destination, dateSent, message, cost) VALUES (";
				//				sql += userid.ToString() + ", ";
				//				sql += "'" + destination +"', ";
				//				sql += "'" + DateTime.Now + "', ";
				//				sql += "'" + message.Replace("'", "''") + "', ";
				//				sql += rate.ToString().Replace(",", ".") + ")";
				//				OleDbHelper.ExecuteUpdate(cnx, sql);
			}
			catch(OleDbException ex)
			{
				throw new ApplicationException("general database error in module AdavSmsImpl.Send", ex);
			}
			finally
			{
				if (cnx != null && cnx.State == ConnectionState.Open)
					cnx.Close();
			}

			return cost;
		}

		#endregion

		private bool IsValidToken(string token)
		{
			try
			{
				int i = (int)OleDbHelper.ExecuteScalar(AdavSmsService.Constants.CONN_STR, "SELECT count(*) FROM Users WHERE token='" + token + "'");
				return i == 1;
			}
			catch (OleDbException ex)
			{
				throw new ApplicationException("general database error in module AdavSmsImpl.IsValidToken", ex);
			}
		}
	}
}
